@@ -92,11 +92,11 @@ fhs_directories = ['/', '/usr', '/usr/share', '/var', '/usr/X11R6',
                    '/usr/man', '/usr/doc', '/usr/bin']
 
 # A map between codenames and suites
-CODENAME2SUITE = {'squeeze': 'oldoldstable',
-                'wheezy': 'oldstable',
-                'jessie': 'stable',
-                'stretch': 'testing',
-                'buster': 'next-testing',
+CODENAME2SUITE = {'wheezy': 'oldoldstable',
+                'jessie': 'oldstable',
+                'stretch': 'stable',
+                'buster': 'testing',
+                'bullseye': 'next-testing',
                 'sid': 'unstable',
                 'experimental': 'experimental'}
 SUITE2CODENAME = dict([(suite, codename) for codename, suite in list(CODENAME2SUITE.items())])
@@ -130,7 +130,7 @@ def search_path_for(filename):
     if d:
         return realpath(filename)
 
-    path = os.environ.get("PATH", os.defpath).split('/')
+    path = os.environ.get("PATH", os.defpath).split(':')
     for d in pathdirs:
         if d not in path:
             path.append(d)
@@ -180,6 +180,13 @@ def search_pipe(searchfile, use_dlocate=True):
     return (pipe, use_dlocate)
 
 
+def get_command_output(cmd):
+    use_shell = False
+    if isinstance(cmd, str) and ' ' in cmd:
+        use_shell = True
+    return subprocess.run(cmd, shell=use_shell, stdout=subprocess.PIPE).stdout.decode(errors='backslashreplace')
+
+
 def query_dpkg_for(filename, use_dlocate=True):
     try:
         x = os.getcwd()
@@ -331,7 +338,7 @@ def get_package_status(package, avail=False):
     statusre = re.compile('Status: ')
     originre = re.compile('Origin: ')
     bugsre = re.compile('Bugs: ')
-    descre = re.compile('Description(?:-.*)?: ')
+    descre = re.compile('Description(?:-[a-zA-Z]+)?: ')
     fullre = re.compile(' ')
     srcre = re.compile('Source: ')
     sectionre = re.compile('Section: ')
@@ -344,6 +351,7 @@ def get_package_status(package, avail=False):
     recommends = []
     suggests = []
     confmode = False
+    descmode = False
     state = ''
 
     try:
@@ -353,10 +361,10 @@ def get_package_status(package, avail=False):
 
     packarg = pipes.quote(package)
     if avail:
-        output = subprocess.getoutput(
-            "COLUMNS=79 dpkg --print-avail %s 2>/dev/null" % packarg)
+        output = get_command_output(
+            "LC_ALL=C.UTF-8 apt-cache show %s 2>/dev/null" % packarg)
     else:
-        output = subprocess.getoutput(
+        output = get_command_output(
             "COLUMNS=79 dpkg --status %s 2>/dev/null" % packarg)
 
     for line in output.split(os.linesep):
@@ -364,12 +372,18 @@ def get_package_status(package, avail=False):
         if not line:
             continue
 
+        if descmode:
+            if line[0] == ' ':
+                fulldesc.append(line)
+            else:
+                descmode = False
+
         if confmode:
             if line[:2] != ' /':
                 confmode = False
             else:
                 # re is used to identify also conffiles with spaces in the name
-                conffiles = conffiles + [re.findall(r' (.+) ([0-9a-f]+).*$', line)[0]]
+                conffiles += re.findall(r' (.+) ([0-9a-f]+).*$', line)
 
         if versionre.match(line):
             (crud, pkgversion) = line.split(": ", 1)
@@ -385,6 +399,7 @@ def get_package_status(package, avail=False):
             (crud, bugs) = line.split(": ", 1)
         elif descre.match(line):
             (crud, desc) = line.split(": ", 1)
+            descmode = True
         elif dependsre.match(line):
             (crud, thisdepends) = line.split(": ", 1)
             # Remove versioning crud
@@ -412,8 +427,6 @@ def get_package_status(package, avail=False):
             src_name = src_name.split()[0]
         elif sectionre.match(line):
             crud, section = line.split(": ", 1)
-        elif desc and line[0] == ' ':
-            fulldesc.append(line)
 
     installed = False
     if status:
@@ -496,7 +509,7 @@ class AvailDB(object):
 
 def get_dpkg_database():
     try:
-        fp = open(STATUSDB)
+        fp = open(STATUSDB, errors="backslashreplace")
         if fp:
             return AvailDB(fp=fp)
     except IOError:
@@ -511,8 +524,8 @@ def get_avail_database():
 
 
 def available_package_description(package):
-    data = subprocess.getoutput('apt-cache show ' + pipes.quote(package))
-    descre = re.compile('^Description(?:-.*)?: (.*)$')
+    data = get_command_output('apt-cache show ' + pipes.quote(package))
+    descre = re.compile('^Description(?:-[a-zA-Z]+)?: (.*)$')
     for line in data.split('\n'):
         m = descre.match(line)
         if m:
@@ -523,7 +536,7 @@ def available_package_description(package):
 def get_source_name(package):
     packages = []
 
-    data = subprocess.getoutput('apt-cache showsrc ' + pipes.quote(package))
+    data = get_command_output('apt-cache showsrc ' + pipes.quote(package))
     packre = re.compile(r'^Package: (.*)$')
     for line in data.split('\n'):
         m = packre.match(line)
@@ -537,7 +550,7 @@ def get_source_package(package):
     retlist = []
     found = {}
 
-    data = subprocess.getoutput('apt-cache showsrc ' + pipes.quote(package))
+    data = get_command_output('apt-cache showsrc ' + pipes.quote(package))
     binre = re.compile(r'^Binary: (.*)$')
     for line in data.split('\n'):
         m = binre.match(line)
@@ -586,7 +599,7 @@ def get_package_info(packages, skip_notfound=False):
     packob = re.compile('^Package: (?P<pkg>.*)$', re.MULTILINE)
     statob = re.compile('^Status: (?P<stat>.*)$', re.MULTILINE)
     versob = re.compile('^Version: (?P<vers>.*)$', re.MULTILINE)
-    descob = re.compile('^Description(?:-.*)?: (?P<desc>.*)$', re.MULTILINE)
+    descob = re.compile('^Description(?:-[a-zA-Z]+)?: (?P<desc>.*)$', re.MULTILINE)
 
     ret = []
     for p in packinfo:
@@ -648,6 +661,8 @@ def get_dependency_info(package, depends, rel="depends on"):
 
     dependencies = []
     for dep in depends:
+        # drop possible architecture qualifier from package names
+        dep = [d.split(':')[0] for d in dep]
         for bit in dep:
             dependencies.append((tuple(dep), bit))
 
@@ -704,7 +719,7 @@ def get_changed_config_files(conffiles, nocompress=False):
             confinfo[filename] = msg
             continue
 
-        filemd5 = subprocess.getoutput('md5sum ' + pipes.quote(filename)).split()[0]
+        filemd5 = get_command_output('md5sum ' + pipes.quote(filename)).split()[0]
         if filemd5 == md5sum:
             continue
 
@@ -732,7 +747,7 @@ DISTORDER = ['oldstable', 'stable', 'testing', 'unstable', 'experimental']
 def get_debian_release_info():
     debvers = debinfo = verfile = warn = ''
     dists = []
-    output = subprocess.getoutput('apt-cache policy 2>/dev/null')
+    output = get_command_output('apt-cache policy 2>/dev/null')
     if output:
         mre = re.compile('\s+(\d+)\s+.*$\s+release\s.*o=(Ubuntu|Debian|Debian Ports),a=([^,]+),', re.MULTILINE)
         found = {}
@@ -776,11 +791,11 @@ def get_debian_release_info():
 
 
 def lsb_release_info():
-    return subprocess.getoutput('lsb_release -a 2>/dev/null') + '\n'
+    return get_command_output('lsb_release -a 2>/dev/null')
 
 
 def get_arch():
-    arch = subprocess.getoutput('COLUMNS=79 dpkg --print-architecture 2>/dev/null')
+    arch = get_command_output('COLUMNS=79 dpkg --print-architecture 2>/dev/null').strip()
     if not arch:
         un = os.uname()
         arch = un[4]
@@ -791,7 +806,7 @@ def get_arch():
 
 
 def get_multiarch():
-    out = subprocess.getoutput('COLUMNS=79 dpkg --print-foreign-architectures 2>/dev/null')
+    out = get_command_output('COLUMNS=79 dpkg --print-foreign-architectures 2>/dev/null')
     return ', '.join(out.splitlines())
 
 
@@ -976,7 +991,7 @@ def parse_config_files():
     for filename in FILES:
         if os.path.exists(filename):
             try:
-                lex = our_lex(open(filename), posix=True)
+                lex = our_lex(open(filename, errors="backslashreplace"), posix=True)
             except IOError as msg:
                 continue
 
@@ -1189,7 +1204,7 @@ def launch_mbox_reader(cmd, url, http_proxy, timeout):
         return
     (fd, fname) = TempFile()
     try:
-        for line in mbox:
+        for line in mbox.splitlines():
             fd.write(line)
         fd.close()
         if cmd is not None:
@@ -1235,7 +1250,7 @@ def exec_and_parse_bugscript(handler, bugscript):
     isattachments = False
     headers = pseudoheaders = text = ''
     attachments = []
-    fp = open(filename)
+    fp = open(filename, errors="backslashreplace")
     for line in fp.readlines():
         # we identify the blocks for headers and pseudo-h
         if line == '-- BEGIN HEADERS --\n':
